<?php
/**
 * @file
 * Form callback for the tracking device identifier form.
 */

/**
 * Callback function for setting the device id.
 *
 * @param object $user
 *   Current device's owner user object.
 *
 * @see nodejs_livetracking_submit_form_submit
 *
 * @see nodejs_livetracking_submit_form_validate
 */
function nodejs_livetracking_settings_form($form, &$form_state, $user) {
  $default = 0;
  $idvalue = '';
  $form_state['uid_form'] = $user->uid;

  $query = db_query("SELECT * FROM {nodejs_livetracking_device} WHERE uid=:uid", array(':uid' => $user->uid))->fetchAll();

  if (count($query) > 0) {
    $default = 1;
    $idvalue = $query[0]->pass;
  }

  $form['app_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Device login'),
    '#description' => t("Your tracking device login is your website username."),
    '#disabled' => TRUE,
    '#prefix' => t("Your tracking device must use the following login and password to get authenticated :"),
    '#value' => $user->name,
  );

  $form['app_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Device password'),
    '#description' => t("Enter your tracking device password. Please note this password will be sent in clear text by your tracking device. DON'T USE YOUR FAVORITE ONE FOR THIS PURPOSE !!"),
    '#default_value' => $idvalue,
  );

  $options = array(
    0 => t("No I disagree."),
    1 => t("Yes I agree"),
  );

  $form['phonegap_disclaimer'] = array(
    '#type' => 'radios',
    '#title' => t("Terms of use"),
    '#options' => $options,
    '#description' => t("By saving this form, I give full authorisation to this website and/or to this service owner to use my localisation data for livetracking and for safety purpose. I also acknowledge and agree that thoses data will be publicly shown."),
    '#default_value' => $default,
  );

  $form['phonegap_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save the form'),
    '#validate' => array('nodejs_livetracking_submit_form_validate'),
    '#submit' => array('nodejs_livetracking_submit_form_submit'),
  );

  return $form;
}

/**
 * Form validation handler for tracking device setting.
 *
 * No blanck uuid.
 * No already used uuid.
 * Must agree terms of use.
 *
 * @see nodejs_livetracking_submit_form_submit
 */
function nodejs_livetracking_submit_form_validate($form, &$form_state) {
  if ($form['app_pass']['#default_value'] == '' && $form_state['values']['phonegap_disclaimer'] != 1) {
    form_set_error('phonegap_disclaimer', t("You must agree with the terms of use to log in your tracking device."));
    return;
  }

  if (strlen($form_state['values']['app_pass']) < 3){
    form_set_error('app_pass', t("Your password must contain at least 4 characters."));
  }
}

/**
 * Form submit handler for tracking device setting.
 *
 * Delete or merge the tracking uuid for this user.
 * Notify the user.
 *
 * @see nodejs_livetracking_submit_form_validate
 */
function nodejs_livetracking_submit_form_submit($form, &$form_state) {
  if ($form_state['values']['phonegap_disclaimer'] != 1) {
    db_delete('nodejs_livetracking_device')
      ->condition('uid', $form_state['uid_form'])
      ->execute();
    drupal_set_message(t("Your tracking device password has been removed. You must agree with the term of use to get it registered again."));
  }
  else {
    db_merge('nodejs_livetracking_device')->key(array('uid' => $form_state['uid_form']))->fields(array(
      'pass' => $form_state['values']['app_pass'],
    ))->execute();
    drupal_set_message(t('Your tracking device has been registered !'));
  }
}
