/**
 * @File
 *
 * Javascript handler for livetracking map.
 *
 * Prepare the map, receive a message from Nodejs and add the position.
 * Toggle connected / disconnected.
 * Show last position.
 */

(function($) {
  var ge;
  var geLoaded = false;
  var overallStyle;
  var autoZoom = true;
  var pointsArray = Array();
  var lastTime;
  var marker;
  var lineString;
  var track;

  Drupal.behaviors.nodejs_livetracking = {
    'attach': function(context, setting) {
      if (context == document) {
        // I grab the style defined form the nodejs_livetracking module to
        // Openlayers
        overallStyle = Drupal.settings.nodejs_livetracking;

        google.load("earth", "1", {
          'callback': function() {
            google.earth.createInstance('map3d', Drupal.behaviors.nodejs_livetracking.initCB, Drupal.behaviors.nodejs_livetracking.failureCB);
          }
        });

        $('#followme').bind('change', function() {
          autoZoom = !autoZoom;
        });

        setInterval(function() {
          $('#tm').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime))
        }, 5000);
      }
    },

    'init' : function() {
      google.earth.createInstance('map3d', Drupal.behaviors.nodejs_livetracking.initCB, Drupal.behaviors.nodejs_livetracking.failureCB);
    },

    'initCB' : function (instance) {
      ge = instance;
      ge.getOptions().setFlyToSpeed(ge.SPEED_TELEPORT);
      ge.getOptions().setTerrainExaggeration(2);

      geLoaded = true;
      // Preparing the marker.
      marker = ge.createPlacemark('');
      marker.setName(Drupal.t("User position"));
      var icon = ge.createIcon('');
      icon.setHref(overallStyle.url + "/" + overallStyle.externalGraphic);
      var style = ge.createStyle('');
      style.getIconStyle().setIcon(icon);
      marker.setStyleSelector(style);
      var point = ge.createPoint('');
      point.setAltitudeMode(ge.ALTITUDE_ABSOLUTE);
      marker.setGeometry(point);
      ge.getFeatures().appendChild(marker);

      track = ge.createPlacemark('');
      lineString = ge.createLineString('');
      lineString.setAltitudeMode(ge.ALTITUDE_ABSOLUTE);
      track.setGeometry(lineString);

      track.setStyleSelector(ge.createStyle(''));
      var lineStyle = track.getStyleSelector().getLineStyle();
      lineStyle.setWidth(parseInt(overallStyle.strokeWidth));
      var kmlColor = Drupal.Nodejs.nodejs_livetracking_hexToKml(overallStyle.strokeColor, overallStyle.strokeOpacity);
      lineStyle.getColor().set(kmlColor);

      ge.getFeatures().appendChild(track);

      // Show the track already stored.
      var pastTrack = Drupal.settings.nodejs_livetracking_storage.past;
      if (pastTrack.length > 1) {
        Drupal.behaviors.nodejs_livetracking.updateInfo(pastTrack[0]);
        Drupal.behaviors.nodejs_livetracking.setLastTime(pastTrack[0].tm);
        for (var i = 0; i < pastTrack.length; i++) {
          Drupal.behaviors.nodejs_livetracking.setTrack(Array(pastTrack[i].lat, pastTrack[i].lon, pastTrack[i].altitude));
        }
        Drupal.behaviors.nodejs_livetracking.setPoint(pointsArray[0]);
        var zoomTo = true;
      }
      ge.getWindow().setVisibility(true);
      if (zoomTo == true) {
        Drupal.behaviors.nodejs_livetracking.zoomToPlacemark();
      }
      $("#fullscreen").removeClass("hidden").click(function(){
        var element = document.getElementById("map3d");
        if(element.requestFullScreen) {
          //Official W3C function.
          element.requestFullScreen();
        }
        else if(element.webkitRequestFullScreen) {
          //Google Chrome function.
          element.webkitRequestFullScreen(Element.ALLOW_KEYBOARD_INPUT);
        }
        else if(element.mozRequestFullScreen) {
          //Firefox function.
          element.mozRequestFullScreen();
        }
        else {
          alert(Drupal.t("Your browser doesn't support HTML5 fullscreen mode. Time to get a new one ;)"));
        }
        return false;
      });
    },

    'failureCB' : function (errorCode) {
    },

    // Set latest marker.
    'setPoint' : function(pointArray) {
      var point = marker.getGeometry();
      point.setLatitude(pointArray[0]);
      point.setLongitude(pointArray[1]);
      point.setAltitude(pointArray[2]);
    },

    'setTrack' : function(point) {
      pointsArray.push(point);
      ge.getFeatures().removeChild(track);
      lineString.getCoordinates().pushLatLngAlt(point[0], point[1], point[2]);
      track.setGeometry(lineString);
      ge.getFeatures().appendChild(track);
    },

    'updateInfo' : function(message) {
      for (var key in message) {
        if (['lat', 'lon', 'precision', 'vario', 'alt', 'sog'].indexOf(key) > -1) {
          $('#' + key).html(message[key]);
        }
      }
    },

    'zoomToPlacemark' : function() {
      // Get the current view.
      var lookAt = ge.getView().copyAsLookAt(ge.ALTITUDE_RELATIVE_TO_GROUND);

      // Set new latitude and longitude values.
      var point = marker.getGeometry();
      lookAt.setLatitude(point.getLatitude());
      lookAt.setLongitude(point.getLongitude());
      lookAt.setRange(5000);

      // Update the view in Google Earth.
      ge.getView().setAbstractView(lookAt);
    },

    'setPresence' : function(bool) {
      if (bool == true) {
        $("#connected").removeClass('hidden');
        $("#disconnected").addClass('hidden');
      }
      else {
        $("#connected").addClass('hidden');
        $("#disconnected").removeClass('hidden');
      }
    },

    'setLastTime' : function(time) {
      lastTime = time;
       $('#tm').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime));
    },

    // Return time ago from timestamp. Copied and adapted from : https://gist.github.com/skattyadz/1285806
    'timeAgo' : function(time) {
      if (time == undefined) {
        return Drupal.t("Unknown");
      }
      var units = [
        { name: Drupal.t("second"), limit: 60, in_seconds: 1 },
        { name: Drupal.t("minute"), limit: 3600, in_seconds: 60 },
        { name: Drupal.t("hour"), limit: 86400, in_seconds: 3600 },
        { name: Drupal.t("day"), limit: 604800, in_seconds: 86400 },
        { name: Drupal.t("week"), limit: 2629743, in_seconds: 604800 },
        { name: Drupal.t("month"), limit: 31556926, in_seconds: 2629743 },
        { name: Drupal.t("year"), limit: null, in_seconds: 31556926 }
      ];

      var diff = (new Date() - new Date(time)) / 1000;
      if (diff < 5) {
        return Drupal.t("just now");
      }
      var i = 0;
      while (unit = units[i++]) {
        if (diff < unit.limit || !unit.limit){
          var diff = Math.floor(diff / unit.in_seconds);
          return diff + " " + unit.name + (diff > 1 ? "s" : "") + " " + Drupal.t("old");
        }
      };
    },
  };

  Drupal.Nodejs.nodejs_livetracking_hexToKml = function (hex, alpha) {
    if (hex.charAt(0) == "#") {
      hex = hex.substring(1,7);
    }

    if (alpha < 1 && alpha > 0) {
      alpha = Math.round(alpha * 10).toString(16);;
    }
    else {
      alpha = 'F';
    }

    var r = hex.substring(0,2);
    var g = hex.substring(2,4);
    var b = hex.substring(4,6);

    return('#' + alpha + '' + alpha + '' + b + '' + g + '' + r);
  };

  Drupal.Nodejs.callbacks.nodejsGeoloc = {
    callback: function (message) {
      if (message.messageType == 'device-auth') {
        Drupal.behaviors.nodejs_livetracking.setPresence(true);
        message['data'] = {
          'body' : Drupal.t('User @id is now here !', {'@id' : message.uid}),
          'subject' : Drupal.t('Device connected')
        };
        Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
      }
      else if (message.messageType == 'device-off') {
        Drupal.behaviors.nodejs_livetracking.setPresence(false);
        message['data'] = {
          'body' : Drupal.t('We just lost user @id !', {'@id' : message.uid}),
          'subject' : Drupal.t('Device disconnected')
        };
        Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
        // There is no position in this message. I just quit now.
        return;
      }

      if (geLoaded == true) {
        Drupal.behaviors.nodejs_livetracking.setPoint(Array(parseFloat(message.position.lat), parseFloat(message.position.lon), parseFloat(message.position.alt)));
        Drupal.behaviors.nodejs_livetracking.setTrack(Array(parseFloat(message.position.lat), parseFloat(message.position.lon), parseFloat(message.position.alt)));
        if (autoZoom == true) {
          Drupal.behaviors.nodejs_livetracking.zoomToPlacemark();
        }
      }
      Drupal.behaviors.nodejs_livetracking.updateInfo(message.position);
      Drupal.behaviors.nodejs_livetracking.setLastTime(message.position.tm);
    },
  };
})(jQuery);
