/**
 * @File
 *
 * Javascript handler for livetracking group map.
 *
 * Prepare the map, receive a message from Nodejs and add the position.
 */

(function($) {
  var ol;
  var pointFeature;
  var pointsArray = Array();
  var line;
  var lineFeature;
  var positionLayer;
  var trackLayer;
  var autoZoom = true;
  var lastTime;

  Drupal.behaviors.nodejs_livetracking = {
    'attach': function(context, setting) {
      if (context == document) {
        ol = $('#openlayers-map').data('openlayers');
        // Catch defined 'livetracking_style' from the api.
        var mapStyle = new OpenLayers.StyleMap(ol.map.styles);
        // Create position and track layer and assigning a livetracking style. to it
        positionLayer = new OpenLayers.Layer.Vector('User position', {styleMap: mapStyle});
        // Add the layer to the map.
        ol.openlayers.addLayer(positionLayer);
        
        var point = new OpenLayers.Geometry.Point(0, 0).transform("EPSG:4326", ol.map.projection);
        pointFeature = new OpenLayers.Feature.Vector(point);
        positionLayer.addFeatures([pointFeature]);
      }
     },

    // Get latest marker.
    'getPoint' : function() {
      return pointFeature;
    },

    'storePoint' : function(point) {
      pointsArray.push(point);
      if (pointsArray.length == 2) {
        line = new OpenLayers.Geometry.LineString(pointsArray);
        lineFeature = new OpenLayers.Feature.Vector(line);
        trackLayer.addFeatures([lineFeature]);
      }
      else if (pointsArray.length > 2) {
        line.addPoint(point);
      }
      trackLayer.redraw();
    },

    // Set latest marker.
    'setPoint' : function(feature) {
      pointFeature = feature;
    },

    // Zoom the feature (Follow).
    'zoomToFeature' : function() {
      if (autoZoom == true){
        ol.openlayers.zoomToExtent(positionLayer.getDataExtent());
      }
    },

  Drupal.Nodejs.callbacks.nodejs_livetracking_tracking = {
    callback: function (message) {
      if (message.destinationChannel != "all_tracking") {
        if (message.messageType == 'device-auth') {
          Drupal.behaviors.nodejs_livetracking.setPresence(true);
          message['data'] = {
            'body' : Drupal.t('User @id is now here !', {'@id' : message.uid}),
            'subject' : Drupal.t('Device connected')
          };
          Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
        }
        else if (message.messageType == 'device-off') {
          Drupal.behaviors.nodejs_livetracking.setPresence(false);
          message['data'] = {
            'body' : Drupal.t('We just lost user @id !', {'@id' : message.uid}),
            'subject' : Drupal.t('Device disconnected')
          };
          Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
          // There is no position in this message. I just quit now.
          return;
        }

        var newLonLat = new OpenLayers.LonLat(message.position.lon, message.position.lat).transform("EPSG:4326", ol.map.projection);
        pointFeature.move(newLonLat);
        positionLayer.redraw();
        Drupal.behaviors.nodejs_livetracking.zoomToFeature();
        var point = new OpenLayers.Geometry.Point(message.position.lon, message.position.lat).transform("EPSG:4326", ol.map.projection);
        Drupal.behaviors.nodejs_livetracking.storePoint(point);
        Drupal.behaviors.nodejs_livetracking.updateInfo(message.position);
        Drupal.behaviors.nodejs_livetracking.setLastTime(message.position.tm);
      }
    },
  };
})(jQuery);
