/**
 * @File
 * Livetracking server extension for nodejs
 *
 * IMPORTANT ! Add this extension name into node.config.js
 * @see nodejs_livetracking_storage.
 */

/**
 * There is many node caching module around. I started with node-cache tryed memory-cache but finally decided to switch to expirable instead.
**/

var Expirable = require('expirable');
var myCache = new Expirable('3 minutes');

// config get from config export.setup
var configuration;

/**
 * Helper function that check for errors in backend response.
 */
var checkErrors = function (error, response, body, type) {
  var authData = false;
  if (error) {
    if (configuration.debug == true) {
      console.log("Error with " + type + " on device request:", error);
    }
    return authData;
  }
  if (response.statusCode == 404) {
    if (configuration.debug == true) {
      console.log('Backend ' + type + ' url not found, full response info:', response);
    }
    return authData;
  }
  try {
    authData = JSON.parse(body);
  }
  catch(exception) {
    if (configuration.debug == true) {
      console.log('Failed to parse ' + type + ' message:', exception);
      console.log('Failed message string: ' + body);
    }
    return authData;
  }

  return authData;
}

/**
 * Handle incoming message entering from route "/client.php"
 *
 *  @todo: I have a bad coding problem here. Duplicate code.
 *  @see : authDeviceCallback variable repeated.
 */
var clientAuth = function(request, reponse) {
  var message = request.query;
  if (message.hasOwnProperty('op') == true && message.hasOwnProperty('user') == true && message.hasOwnProperty('pass') == true) {
    var authMes = {
      messageType: 'device-auth',
      user : message.user,
      pass : message.pass,
    };
    // => Authenticate the device.
    // This is where I just repeat the function defined in authDeiceCallback below. I can't get reponse variable to be in otherwise !! 
    // I need this reponse variable to be there in order to return the uid to the device.
    configuration.sendMessageToBackend(authMes, function (error, response, body) {
      var authData = checkErrors(error, response, body, 'authentication');
      if (authData == false) {
        return;
      }
      // The first element is the key. Let's shift !
      console.log(authData);
      var key = authData.shift();
      myCache.set(key, authData);
      myCache.once(key + ':removed', function(expired) {
      // The expired boolean tells if the key was removed because it was expired or if it was a manual removal
        if (expired == true) {
          var offMes = {
            messageType: 'device-off',
            uuid: key,
            uid: authData[0],
            name: authData[2],
          };
          // => Set device Offline.
          configuration.sendMessageToBackend(offMes);
        }
      });
      // Here I send the uid result to the device.
      reponse.send(authData[0]);
    });
  }
}

/**
 * Handle authentication device call response.
 *
 *  @todo : This could be merged with above somehow.
 *  @see : clientAuth function. 
 */
var authDeviceCallback = function (error, response, body) {
  // console.log(reponse); // Won't work -> this is my problem. See above !
  var authData = checkErrors(error, response, body, 'authentication');
  if (authData == false) {
    return;
  }

  // The first element is the key. Let's shift !
  var key = authData.shift();
  myCache.set(key, authData);
  cacheRemoveEvent(key, key, authData);
  return authData[0];
}

/**
 * Handle incoming message entering from route "/" or from /track.php
 */
var trackAuth = function(request, reponse) {
  // Parsing the response to get JSON.
  var message = request.query;
  if (configuration.debug == true) {
    console.log(message);
  }
  if (message.hasOwnProperty('leolive') == true) {
    var leolive = message.leolive;
    var deviceIsCached = detectCached(leolive, message);
    //console.log("Cached = " + deviceIsCached);
    if (deviceIsCached == false){
      cacheDevice(leolive, message);
    }

    cachedData = getCachedData(leolive, message);
    //console.log("cache = " + cachedData);
    if (cachedData == "Pending Authentication") {
      return;
    }

    if (leolive == 1 || leolive == 4) {
      posMes = {
        messageType: 'device-pos',
        position : message,
        //channel : cachedData[1],
        uid : cachedData[0],
      }
      // => Send the position to all channels.
      for (var i=0; i < cachedData[1].length; i++) {
        if (cachedData[1][i] != "all_tracking") {
          posMes.channel = cachedData[1][i]; 
          sendMessageToTokenChannel(posMes);
        }
      }

      configuration.sendMessageToBackend(posMes, function (error, response, body) {
        console.log('Position sent to backend');
      });
    }

    if (leolive == 3) {
      myCache.expire(message.sid, '2 seconds');
    }
  reponse.send("ok");   
  }
}

/**
 * Helper function that detect if a device is already cached. 
 */
function detectCached(leolive, message){
  if (leolive == 1){
    return myCache.has(message.user + '-' + message.pass) ? true : false;
  }
  else {
    return myCache.has(message.sid) ? true : false;
  }
  
  return false;
}

/**
 * Helper function that will cache a device if it's not already cached.
 */
function cacheDevice(leolive, message){
  if (leolive == 1){
    myCache.set(message.user + '-' + message.pass, "Pending Authentication");
    var authMes = {
      messageType: 'device-auth',
      user : message.user,
      pass : message.pass
    };
    // => Authenticate the device.
    configuration.sendMessageToBackend(authMes, authDeviceCallback);
  }
  else if (leolive == 2) {
    var value = myCache.get(message.user + '-' + message.pass);
    myCache.set(message.sid, value);
    cacheRemoveEvent(message.sid, message.user + '-' + message.pass, value);
    myCache.remove(message.user + '-' + message.pass);
    // I'm sure a new track is starting (session)... Delete (or soon save) past track elsewhere.
    deletePastTrack(value[0]);
  }
}

/**
 * Helper function that delete past track from an authenticated user.
 *
 *  @todo : Save it as you want it before simply deleting it. POSTGIS !!
 */
function deletePastTrack(uid) {
  // => Deleting past position;
  if (storage == true) {
    mongo.trackingModel.remove({uid: uid}, function(err) {
      if (err) {
        if (configuration.debug == true) {
          console.log("Error : " + err);
        }
      }
      else {
        if (configuration.debug == true) {
          console.log(message.user + '-' + message.pass + " Points erased from Mongo");
        }
      }
    });
  }
}

/**
 * Helper function that add an expirable event to a cache entry.
 */
function cacheRemoveEvent(key, uuid, value) {
  myCache.once(key + ':removed', function(expired) {
    // The expired boolean tells if the key was removed because it was expired or if it was a manual removal
    if (expired == true) {
      var offMes = {
        messageType: 'device-off',
        uuid: uuid,
        uid: value[0],
        name: value[2],
      };
      // => Set device Offline.
      configuration.sendMessageToBackend(offMes);
    }
  });
}

/**
 * Helper function that will grab cached data.
 */
function getCachedData(leolive, message) {
  if (leolive == 1) {
    return myCache.get(message.user + '-' + message.pass);
  }
  else {
    return myCache.get(message.sid);
  }
}

/**
 * Send message to contentChannel via Nodejs.
 */
function sendMessageToTokenChannel(message) {
  if (!message.hasOwnProperty('channel')) {
    console.log('publishMessageToContentChannel: An invalid message object was provided.');
    return;
  }
  if (!configuration.tokenChannels.hasOwnProperty(message.channel)) {
    console.log('publishMessageToContentChannel: The channel "' + message.channel + '" doesn\'t exist.');
    return;
  }

  for (var socketId in configuration.tokenChannels[message.channel].sockets) {
    configuration.publishMessageToClient(socketId, message);
  }
}

/**
 * Handle incoming message entering from route "nodejs/livetracking/sendToMultipleContentChannelUrl"
 *
 * This function is a clone of server.js publishMessageToContentChannel.
 * It only allow to send to multiple declared channel at once.
 *
 * @see server.js
 */
function sendToMultipleContentChannel(request, response) {
  var sentCount = 0, requestBody = '';
  request.setEncoding('utf8');
  request.on('data', function (chunk) {
    requestBody += chunk;
  });
  request.on('end', function () {
    try {
      var message = JSON.parse(requestBody);
      if (configuration.settings.debug) {
        console.log('sendToMultipleContentChannel: message', message);
      }
    }
    catch (exception) {
      console.log('sendToMultipleContentChannel: Invalid JSON "' + requestBody + '"', exception);
      response.send({error: 'Invalid JSON, error: ' + exception.toString()});
      return;
    }
    if (!message.hasOwnProperty('channel')) {
      console.log('sendToMultipleContentChannel: An invalid message object was provided.');
      response.send({error: 'Invalid message'});
      return;
    }

    for (var i=0; i < message.channel.length; i++) {
      var channel = message.channel[i];
      if (!configuration.tokenChannels.hasOwnProperty(channel)) {
        continue;
      }
      message.destinationChannel = channel;
      for (var socketId in configuration.tokenChannels[channel].sockets) {
        configuration.publishMessageToClient(socketId, message);
      }
    }
    response.send({sent: 'sent'});
  });
}

// Here I'll process the message depending on which trackeur type it is.
var processPosition = function(mess) {
  // If it's from PhoneGap... Don't touch.
  // else process mess to get a PhoneGap like message !
  // return message;
}

// Define a route that will receive tracking message.
exports.routes = Array(
  {path : '/', type : 'get', handler : trackAuth},
  {path : '/track.php', type: 'get', handler : trackAuth},
  {path : '/client.php', type: 'get', handler : clientAuth},
  {path: '/nodejs/livetracking/sendToMultipleContentChannelUrl', type: 'post', handler: sendToMultipleContentChannel}
);

exports.setup = function(config) {
  configuration = config;
}

exports.mongo = function(mong) {
  mongo = mong;
  storage = true;
}
